#ifndef __IRVINE_LCD_DATA_H__
#define __IRVINE_LCD_DATA_H__

#include <linux/device.h>
#include <linux/amba/bus.h>
#include <linux/amba/bcmring_clcd.h>
#include <plat/irvine-lcd.h>

struct clcd_panel qvga888 = 
{
	.mode = {
		.name		= "BCM-LCD",
		.refresh	= 60,
		.xres		= 320,
		.yres		= 240,
		.pixclock	= 111000,	/* 90090000 for 60 Hz, this driver wants in picoseconds=111000ps */
		.left_margin	= 0x16,		/* horizontal back porch */
		.right_margin	= 0x4,		/* horizontal front porch */
		.upper_margin	= 0x12,		/* vertical back porch */
		.lower_margin	= 0x9,		/* vertical front porch */
		.hsync_len	= 0x2d, 	/* horizontal pulse width */
		.vsync_len	= 0x01,		/* vertical pulse width */
		.sync		= 0,		/* horizontal and vertical both active low */
		.vmode		= FB_VMODE_NONINTERLACED,
	},
	.width	= -1,
	.height	= -1,
	.tim2	= 0x013f78c0,
	.tim3	= 0,
	.cntl	= CNTL_LCDPWR | CNTL_LCDVCOMP(1) | CNTL_LCDTFT  | CNTL_LCDBPP24 | CNTL_BGR,
	.bpp	= 32,    /* we actually only display 24 bpp but this is the setting the driver needs */
};

struct clcd_panel qvga565 = 
{
	.mode = {
		.name		= "BCM-LCD",
		.refresh	= 60,
		.xres		= 320,
		.yres		= 240,
		.pixclock	= 111000,	/* 90090000 for 60 Hz, this driver wants in picoseconds=111000ps */
		.left_margin	= 0x16,		/* horizontal back porch */
		.right_margin	= 0x4,		/* horizontal front porch */
		.upper_margin	= 0x12,		/* vertical back porch */
		.lower_margin	= 0x9,		/* vertical front porch */
		.hsync_len	= 0x2d,		/* horizontal pulse width */
		.vsync_len	= 0x01,		/* vertical pulse width */
		.sync		= 0,		/* horizontal and vertical both active low */
		.vmode		= FB_VMODE_NONINTERLACED,
	},
	.width	= -1,
	.height	= -1,
	.tim2	= 0x013f78c0,
	.tim3	= 0,
	.cntl	= CNTL_LCDPWR | CNTL_LCDVCOMP(1) | CNTL_LCDTFT  | CNTL_LCDBPP16_565 | CNTL_BGR,
	.bpp	= 16,   
};

// Without PLL running on 26MHz TCXO
struct clcd_panel wqvga565_480x272_NoPLL = 
{
	.mode = {
		.name		= "BCM-LCD",	/* for 480x272 */
		.refresh	= 60,
		.xres		= 480,
		.yres		= 272,
		.pixclock	= 76923, 	/* picoseconds=76923ps */
		.left_margin	= 35,		/* horizontal back porch */
		.right_margin	= 35,		/* horizontal front porch */
		.upper_margin	= 20,		/* vertical back porch */
		.lower_margin	= 20,		/* vertical front porch */
		.hsync_len	= 31,		/* horizontal pulse width */
		.vsync_len	= 5,		/* vertical pulse width */
		.sync		= 0,		/* horizontal and vertical both active low */
		.vmode		= FB_VMODE_NONINTERLACED,
	},
	.width	= -1,
	.height	= -1,
	.tim2	= 0x01df1800,
	.tim3	= 0,
	.cntl	= CNTL_LCDPWR | CNTL_LCDVCOMP(1) | CNTL_LCDTFT  | CNTL_LCDBPP16_565 | CNTL_BGR,
	.bpp	= 16,   
};

// With PLL running on 26MHz TCXO
static struct clcd_panel wqvga565_480x272_WithPLL = 
{
	.mode = 
	{
		.name		= "BCM-LCD",	/* for 480x272 */
		.refresh	= 60,
		.xres		= 480,
		.yres		= 272,
		.pixclock	= 96154,	/* picoseconds=769923ps */
		.left_margin	= 6,		/* horizontal back porch */
		.right_margin	= 6,		/* horizontal front porch */
		.upper_margin	= 23,		/* vertical back porch */
		.lower_margin	= 23,		/* vertical front porch */
		.hsync_len	= 5,		/* horizontal pulse width */
		.vsync_len	= 6,		/* vertical pulse width */
		.sync		= 0,		/* horizontal and vertical both active low */
		.vmode		= FB_VMODE_NONINTERLACED,
	},
	.width	= -1,
	.height	= -1,
	.tim2	= 0x01df18c3,	/* 0x013f78c0 for 340x280 */
	.tim3	= 0,
	.cntl	= CNTL_LCDPWR | CNTL_LCDVCOMP(1) | CNTL_LCDTFT  | CNTL_LCDBPP16_565 | CNTL_BGR,
	.bpp	= 16,   
};

static struct clcd_panel wvga565_800x480_NoPLL = {
	.mode		= {
		.name		= "BCM-LCD",	/* for 800x480 */
		.refresh	= 60,
		.xres		= 800,
		.yres		= 480,
		.pixclock	= 111111, 	/* 90090000 for 60 Hz, this driver wants in picoseconds=111000ps */
		.left_margin	= 0x42,		/* horizontal back porch */
		.right_margin	= 0x42, 	/* horizontal front porch */
		.upper_margin	= 15,		/* vertical back porch */
		.lower_margin	= 15,		/* vertical front porch */
		.hsync_len	= 0x36,		/* horizontal pulse width */
		.vsync_len	= 10,		/* vertical pulse width */
		.sync		= 0,		/* horizontal and vertical both active low */
		.vmode		= FB_VMODE_NONINTERLACED,
	},
	.width	= -1,
	.height	= -1,
	.tim2	= 0x031F18C4,	/* 0x013f78c0 for 800x480 */
	.tim3	= 0,
	.cntl	= CNTL_LCDPWR | CNTL_LCDVCOMP(1) | CNTL_LCDTFT  | CNTL_LCDBPP16_565 | CNTL_BGR,
	.bpp	= 16,   
};

struct clcd_panel wvga565_800x480_WithPLL = 
{
	.mode = 
	{
		.name		= "BCM-LCD",	/* for 800x480 */
		.refresh	= 60,
		.xres		= 800,
		.yres		= 480,
		.pixclock	= 111111,	/* 90090000 for 60 Hz which this messed up driver wants in picoseconds=111000ps */
		.left_margin	= 0x42,   /* horizontal back porch */
		.right_margin	= 0x42,   /* horizontal front porch */
		.upper_margin	= 15,   /* vertical back porch */
		.lower_margin	= 15,   /* vertical front porch */
		.hsync_len	= 0x36,     /* horizontal pulse width */
		.vsync_len	= 10,       /* vertical pulse width */
		.sync		= 0,        /* horizontal and vertical both active low */
		.vmode		= FB_VMODE_NONINTERLACED,
	},
	.width	= -1,
	.height	= -1,
	.tim2	= 0x031F18C4,	/* 0x013f78c0 for 800x480 */
	.tim3	= 0,
	.cntl	= CNTL_LCDPWR | CNTL_LCDVCOMP(1) | CNTL_LCDTFT  | CNTL_LCDBPP16_565 | CNTL_BGR,
	.bpp	= 16,   
};

static struct clcd_panel LMS500__WithPLL = 
{
	.mode = 
	{
		.name		= "LMS500",
		.refresh	= 60,		/* probably ignored, as it is bogus for all listed panels */
		.xres		= 480,
		.yres		= 272,
		.pixclock	= 90909,
		.left_margin	= 1+38,		/* horizontal back porch */
		.right_margin	= 1+11,		/* horizontal front porch */
		.upper_margin	= 1+5,		/* vertical back porch */
		.lower_margin	= 1+3,		/* vertical front porch */

		.hsync_len	= 1,		/* horizontal pulse width */
		.vsync_len	= 6,		/* vertical pulse width */
		.sync		= 0,		/* horizontal and vertical both active low */
		.vmode		= FB_VMODE_NONINTERLACED,
	},
	.width		= -1,
	.height		= -1,
	.tim2       	= 0x01df18c3,
	.tim3		= 0,
	.cntl		= CNTL_LCDPWR | CNTL_LCDVCOMP(1) | CNTL_LCDTFT  | CNTL_LCDBPP16_565 | CNTL_BGR,
	.bpp		= 16,   
};

/* Copied from 'wqvga565_480x272_WithPLL' above */
/* Contains the modeline as used on the Treviso PND */
struct clcd_panel LMS430_Treviso_WithPLL = 
{
	.mode = {
		.name		= "LMS430",
		.refresh	= 60,		/* probably ignored, as it is bogus for all listed panels */
		.xres		= 480,
		.yres		= 272,
		.pixclock	= 90909,

		.left_margin	= 46,		/* horizontal back porch */
		.right_margin	= 8,		/* horizontal front porch */
		.upper_margin	= 1,		/* vertical back porch */
		.lower_margin	= 3,		/* vertical front porch */

		.hsync_len	= 42,		/* horizontal pulse width */
		.vsync_len	= 11,		/* vertical pulse width */
		.sync		= 0,		/* horizontal and vertical both active low */
		.vmode		= FB_VMODE_NONINTERLACED,
	},
	.width		= -1,
	.height		= -1,
	.tim2       	= 0x01df18c3,
	.tim3		= 0,
	.cntl		= CNTL_LCDPWR | CNTL_LCDVCOMP(1) | CNTL_LCDTFT  | CNTL_LCDBPP16_565 | CNTL_BGR,
	.bpp		= 16,   
};

struct clcd_panel LQ043T3DW02_Treviso_WithPLL = 
{
	.mode = 
	{
		.name		= "LQ043T3DW02",
		.refresh	= 60,		/* probably ignored, as it is bogus for all listed panels */
		.xres		= 480,
		.yres		= 272,
		.pixclock	= 90909,
		.left_margin	= 3,		/* horizontal back porch */
		.right_margin	= 2,		/* horizontal front porch */
		.upper_margin	= 2,		/* vertical back porch */
		.lower_margin	= 2,		/* vertical front porch */
		.hsync_len	= 13,		/* horizontal pulse width */
		.vsync_len	= 2,		/* vertical pulse width */
		.sync		= 0,		/* horizontal and vertical both active low */
		.vmode		= FB_VMODE_NONINTERLACED,
	},
	.width		= -1,
	.height		= -1,
	.tim2		= 0x01df38c3,
	.tim3		= 0,
	.cntl		= CNTL_LCDPWR | CNTL_LCDVCOMP(1) | CNTL_LCDTFT  | CNTL_LCDBPP16_565 | CNTL_BGR,
	.bpp		= 16,   
};

struct clcd_device qvga888_dev = 
{
	.panel		= &qvga888,
	.lcd_mdiv	= 0x00000018,
};

struct clcd_device qvga565_dev = 
{
	.panel		= &qvga565,
	.lcd_mdiv	= 0x00000018,
};

struct clcd_device wqvga565_480x272_NoPLL_dev = 
{
	.panel		= &wqvga565_480x272_NoPLL,
	.lcd_mdiv	= 0x00000018,
};

struct clcd_device wqvga565_480x272_WithPLL_dev = 
{
	.panel		= &wqvga565_480x272_WithPLL,
	.lcd_mdiv	= 0x00000018,
};

struct clcd_device wvga565_800x480_NoPLL_dev = 
{
	.panel		= &wvga565_800x480_NoPLL,
	.lcd_mdiv	= 0x00000008,
};

struct clcd_device wvga565_800x480_WithPLL_dev = 
{
	.panel		= &wvga565_800x480_WithPLL,
	.lcd_mdiv	= 0x00000008,
};

struct clcd_device LMS430_Treviso_WithPLL_dev = 
{
	.panel		= &LMS430_Treviso_WithPLL,
	.lcd_mdiv	= 0x00000018,
};

struct clcd_device LMS500__WithPLL_dev = 
{
	.panel		= &LMS500__WithPLL,
	.lcd_mdiv	= 0x00000018,
};

struct clcd_device LQ043T3DW02_Treviso_WithPLL_dev = 
{
	.panel		= &LQ043T3DW02_Treviso_WithPLL,
	.lcd_mdiv	= 0x00000018,
};

#endif

